

##############################################################################
# eltest2step is a function to search the minimum of -2LLR by Golden Section Search method
# eltest2step calls both eltest2u and eltest2v and returns the minimum summation of -2LLR
# from eltest2u and eltest2v
# nx is the sample size of X
# x is a non diseased sample
# ab is a vector with 2 elements for the X range of the partial AUC. ab[1] < ab[2]
# ny is the sample size of Y
# y is a diseased sample
# p1 is the lower False Positive Rate to be included in the Partial AUC
# p2 is the higher False Positive Rate to be included in the Partial AUC
# truepauc is the Partial AUC to be tested
##############################################################################
eltest2step <- function(x, ab= vector("numeric",2), y, p1, p2, truepauc, tol=.001, eps){

  nx <- length(x)
  ny <- length(y)
  
  est.pauc <- rep(1/nx, nx)%*% mean.pauc(x, ab[1], ab[2], eps, y)%*% rep(1/ny, ny)
  
  search.low <- ifelse((min(est.pauc, truepauc)-0.1) >0, (min(est.pauc, truepauc)-0.1), 0)
  
  search.high <- ifelse((max(est.pauc, truepauc)+0.1) <1 , (max(est.pauc, truepauc)+0.1), 1)
  
  grat <- (sqrt(5)-1)/2
  
  d <- grat * (search.high - search.low)
  
  x2 <- search.low + d
  
  x1 <- search.high - d
  
  res1 <- eltest2u(ab, x.sample=x, mu=c(1-p2, 1-p1, x1), y, vvec=rep(1/ny, ny), eps)
  
  res2 <- eltest2v(ab, x.sample=x, mu=truepauc, y, res1$uvec/nx, eps)
  
  fun.x1 <-  res1$"-2LLR" + res2$"-2LLR"
  
  res1 <- eltest2u(ab, x.sample=x, mu=c(1-p2, 1-p1, x2), y, vvec=rep(1/ny, ny), eps)
  
  res2 <- eltest2v(ab, x.sample=x, mu=truepauc, y, res1$uvec/nx, eps)
  
  fun.x2 <-  res1$"-2LLR" + res2$"-2LLR"
  
  err <- 100
  
  numit <- 0
  
  while (err > tol) {
    
    numit <- numit + 1
    
    if (fun.x1 < fun.x2){
      
      xopt <- x1
      
      err <- (1 - grat) * abs((search.high - search.low)/xopt)
      
      if (err > tol) {
        
        search.high <- x2
        
        x2 <- x1
        
        fun.x2 <- fun.x1
        
        d <- grat * (search.high - search.low)
        
        x1 <- search.high - d
        
        res1 <- eltest2u(ab, x.sample=x, mu=c(1-p2, 1-p1, x1), y, vvec=rep(1/ny, ny), eps)
        
        res2 <- eltest2v(ab, x.sample=x, mu=truepauc, y, res1$uvec/nx, eps)
        
        fun.x1 <-  res1$"-2LLR" + res2$"-2LLR"
        
        
      }
    }
    
    else {
      
      xopt <- x2
      
      err <- (1 - grat) * abs((search.high - search.low)/xopt)
      
      if (err > tol) {
        
        search.low <- x1
        
        x1 <- x2
        
        fun.x1 <- fun.x2
        
        d <- grat * (search.high - search.low)
        
        x2 <- search.low + d 
        
        res1 <- eltest2u(ab, x.sample=x, mu=c(1-p2, 1-p1, x2), y, vvec=rep(1/ny, ny), eps)
        
        res2 <- eltest2v(ab, x.sample=x, mu=truepauc, y, res1$uvec/nx, eps)
        
        fun.x2 <-  res1$"-2LLR" + res2$"-2LLR"
        
      }
      
    } 
    
  }
  
  res1 <- eltest2u(ab, x.sample=x, mu=c(1-p2, 1-p1, xopt), y, vvec=rep(1/ny, ny), eps)
  
  res2 <- eltest2v(ab, x.sample=x, mu=truepauc, y, res1$uvec/nx, eps)
  
  return("-2LLR"= res1$"-2LLR" + res2$"-2LLR")
  
}

