
####################################################################################################
# eltest2step.all is a function to search the minimum of -2LLR by Golden Section Search method
# eltest2step.all calls both eltest2u.all and eltest2v.all and returns the minimum summation of -2LLR
# from eltest2u.all and eltest2v.all
# nx is the sample size of X
# x is a non diseased sample
# ny is the sample size of Y
# y is a diseased sample
# true is the AUC to be tested
####################################################################################################


eltest2step.all <- function(true, x, y, tol=0.001){
  
  nx <- length(x)
  ny <- length(y)
  est.pauc <- rep(1/nx, nx)%*% mean.whole(x,  y)%*% rep(1/ny, ny)
  
  search.low <- ifelse((min(est.pauc, true)-0.1) >0, (min(est.pauc, true)-0.1), 0)
  
  search.high <- ifelse((max(est.pauc, true)+0.1) <1 , (max(est.pauc, true)+0.1), 1)
  
  grat <- (sqrt(5)-1)/2
  
  d <- grat * (search.high - search.low)
  
  x2 <- search.low + d
  
  x1 <- search.high - d
  
  res1 <- eltest2u.all(x.sample=x, mu=x1, y, vvec=rep(1/ny, ny))
  res2 <- eltest2v.all(x.sample=x, mu=true, y, res1$uvec/nx)
  
  fun.x1 <-  res1$"-2LLR" + res2$"-2LLR"
  
  res1 <- eltest2u.all(x.sample=x, mu=x2, y, vvec=rep(1/ny, ny))
  res2 <- eltest2v.all(x.sample=x, mu=true, y, res1$uvec/nx)
  
  fun.x2 <-  res1$"-2LLR" + res2$"-2LLR"
  
  err <- 100
  
  numit <- 0
  
  while (err > tol) {
    
    numit <- numit + 1
    
    if (fun.x1 < fun.x2){
      
      xopt <- x1
      
      err <- (1 - grat) * abs((search.high - search.low)/xopt)
      
      if (err > tol) {
        
        search.high <- x2
        
        x2 <- x1
        
        fun.x2 <- fun.x1
        
        d <- grat * (search.high - search.low)
        
        x1 <- search.high - d
        
        
        res1 <- eltest2u.all(x.sample=x, mu=x1, y, vvec=rep(1/ny, ny))
        res2 <- eltest2v.all(x.sample=x, mu=true, y, res1$uvec/nx)
        
        fun.x1 <-  res1$"-2LLR" + res2$"-2LLR"
        
      }
    }
    
    else {
      
      xopt <- x2
      
      err <- (1 - grat) * abs((search.high - search.low)/xopt)
      
      if (err > tol) {
        
        search.low <- x1
        
        x1 <- x2
        
        fun.x1 <- fun.x2
        
        d <- grat * (search.high - search.low)
        
        x2 <- search.low + d 
        
        res1 <- eltest2u.all(x.sample=x, mu=x2, y, vvec=rep(1/ny, ny))
        res2 <- eltest2v.all(x.sample=x, mu=true, y, res1$uvec/nx)
        
        fun.x2 <-  res1$"-2LLR" + res2$"-2LLR"
        
      }
      
    }
  }
  res1 <- eltest2u.all(x.sample=x, mu=xopt, y, vvec=rep(1/ny, ny))
  res2 <- eltest2v.all(x.sample=x, mu=true, y, res1$uvec/nx)
  fun.xopt <-  res1$"-2LLR" + res2$"-2LLR"
  
  return(list('-2LLR'=fun.xopt, Pval = 1 - pchisq(fun.xopt, df=1)))
}


