\name{find.UL}
\alias{find.UL}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Find the Wilks Confidence Interval from the Given (empirical) Likelihood Ratio Function
}
\description{
This program uses simple search to find the upper and lower (Wilks) confidence limits based on the
-2 log likelihood ratio, which the required input fun is supposed to supply.\\
Basically, starting from MLE, we search on both directions, by step away from MLE, until we find
values that have -2LLR = level. (the value of -2LLR at MLE is supposed to be zero.)
At current implimentation, only handles one dimesional parameter, i.e. only confidence intervals,
not confidence regions.
}
\usage{
find.UL(step = 0.01, initStep = 0, fun, MLE, level = 3.84, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{step}{
a positive number. The starting step size of the search. Reasonable value should
be about 1/5 of the SD of MLE.
}
  \item{initStep}{
a nonnegative number. The first step size of the search. Sometimes, you may
want to put a larger innitStep to speed the search.
}
  \item{fun}{
a function that returns "-2LLR", which is the -2 log (empirical) likelihood ratio. 

For pAUC/AUC: x is a non diseased sample; y is a diseased sample; true is the pAUC/AUC to be tested.

1. fun=neighb.xy(sp12 = vector("numeric",2), x, y, true, tol=.001, eps) 
for pAUC at [p1, p2] (p1 and p2 are false positve rates and p1 < p2) and sp12 = (p1, p2).

An example for pAUC at (0.05, 0.55):
nx <- 50 rate.x <- 1 ny <- 50 rate.y <- 0.25 p1 <- 0.05 p2 <- 0.55 eps <- 1/nx 
x <- rexp(nx, rate.x)
y <- rexp(ny, rate.y)
est <- est.pAUC(x, y, p=c(p1, p2), eps=eps)
find.UL(step=0.01, fun=neighb.xy, sp12=c(p1, p2), x=x, y=y, MLE=est, eps=eps).

2. fun=neighb.xy3p12(vp, x, y, true, tol = 0.001, eps, ...), 
for pAUC at (p, 1] or [0, p). 
vp = (p, 1) or vp = (0, p), which is chosen based on the pAUC at (p, 1] or [0, p) accordingly'

An example for pAUC at [0, 0.45):
nx <- 50
ny <- 40
p1 <- 0
p2 <- 0.45
x <- rnorm(nx)
y <- rnorm(ny, 2, 1)
eps <- 1/nx
est <- est.pAUC(x, y, p=c(p1, p2), eps=eps)
find.UL(step=0.01, fun=neighb.xy3p12, vp=c(p1, p2), MLE=est, x=x, y=y, eps=eps).

An example for pAUC at (0.45, 1]:
nx <- 100
ny <- 80
p1 <- 0.45
p2 <- 1
x <- rnorm(nx)
y <- rnorm(ny, 2, 1)
eps <- 1/nx
est <- est.pAUC(x, y, p=c(p1, p2), eps=eps)
find.UL(step=0.01, fun=neighb.xy3p12, vp=c(p1, p2), MLE=est, x=x, y=y, eps=eps).

3. fun=eltest2step.all(true, x, y, tol) for AUC. 

An example for AUC:
nx <- 40
ny <- 30
x <- rnorm(nx)
y <- rnorm(ny, 1, 1)
est <- est.pAUC(x, y, p=c(0, 1))
find.UL(step=0.01, fun=eltest2step.all, MLE=est, x=x, y=y).


For trimmed/Lorenz Curves: x is a sample; true is the trimmed/truncated mean to be tested.

1. fun=neighb(sp12 = vector("numeric",2), x, true, eps). for trimmed mean at [p1, p2] (0 < p1 < p2 < 1), where sp12 = (p1, p2). 

An example of trimmed mean at (0.21, 0.79):
nsize <- 50
p1<- 0.21
p2<- 0.79
x <- rnorm(nsize)
est <- est.tr.mean(x, p=c(p1, p2), eps=1/nsize)
find.UL(step=0.01, fun=neighb, sp12=c(p1, p2), MLE=est, x=x, eps=1/nsize).

2. fun=neighb.p((p, x, true, eps)) for truncated mean at $[0, p] (0 < p < 1)$. 

An example of trunated mean/Generalized Lorenz Curve at (0, 0.89):
nsize <- 50
p<- 0.89
x <- rexp(nsize, rate=0.25)
est <- est.tr.mean(x, p, eps=1/nsize)
find.UL(step=0.001, fun=neighb.p, p=p, MLE=est, x=x, eps=1/nsize)

An example of Lorenz Curve at (0, 0.89):
nsize <- 500
p<- 0.89
x <- rexp(nsize, rate=0.25)
est <- est.tr.mean(x, p, eps=1/nsize)/mean(x)
find.UL(step=0.001, fun=neighb.p.lc, p=p, MLE=est, x=x, eps=1/nsize)


In all the above cases, eps is a smoothing parameter. 
tol is the maximum distance between the current guess and the current range on 
Golden Section Search Optimization method.
}
  \item{MLE}{
The MLE of the parameter. No need to be exact, as long as it is inside the confidence interval
}
  \item{level}{
an optional positive number, controls the confidence level. Default to 3.84 =
chisq(0.95, df=1). Change to 2.70=chisq(0.90, df=1) to get a 90\% confidence
interval
}
  \item{\dots}{
additional arguments, if any, to pass to fun
}
}
\details{
Basically we repeatedly testing the value of the parameter, until we find those which the -2 log
likelihood value is equal to 3.84 (or other level, if set differently).\\
If there is no value exactly equal to 3.84, we stop at the value which result a -2 log likelihood just
below 3.84. (as in the discrete case, like quantiles.)
}
\value{
  \item{Low}{the lower limit of the confidence interval.}
  \item{Up}{the upper limit of the confidence interval.}
  \item{FstepL}{the final step size when search lower limit. An indication of the precision.}
  \item{FstepU}{Ditto. An indication of the precision of the upper limit.}
  \item{Lvalue}{The -2LLR value of the final Low value. Should be approximately equal to level.
If larger than level, than the confidence interval limit Low is wrong.}
\item{Uvalue}{Ditto. Should be approximately equa to level.}
}
\references{
The Paritial Area Under ROC Curves by Empirical Likelihood Method
}
\author{
Modify Mai Zhou's function 'findUL' in package 'emplik' by Yumin Zhao
}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{

### An example of trunated mean at (0, 0.89)
set.seed(123)
nsize <- 50
p<- 0.89
x <- rnorm(nsize)
est <- est.tr.mean(x, p, eps=1/nsize)
find.UL(step=0.001, fun=neighb.p, p=p, MLE=est, x=x, eps=1/nsize)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
%\keyword{ ~kwd1 }
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
